// Copyright 2017 Google Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include <iostream>
#include <string>
#include <vector>

#include "sling/base/flags.h"
#include "sling/base/init.h"
#include "sling/base/logging.h"
#include "sling/frame/store.h"
#include "sling/frame/serialization.h"
#include "sling/nlp/document/document.h"
#include "sling/nlp/parser/parser.h"
#include "sling/stream/file-input.h"

DEFINE_string(conll, "", "CONLL input file");
DEFINE_string(output, "", "Output directory for SLING files");
DEFINE_string(parser, "", "Flow file for parser");

using namespace sling;
using namespace sling::nlp;

int main(int argc, char *argv[]) {
  InitProgram(&argc, &argv);

  // Load parser.
  Store commons;
  Parser parser;
  if (!FLAGS_parser.empty()) {
    parser.Load(&commons, FLAGS_parser);
  }
  commons.Freeze();

  // Convert input corpus.
  FileInput in(FLAGS_conll);
  std::vector<string> words;
  string line;
  int sentno = 0;
  while (in.ReadLine(&line)) {
    if (line.empty() || line[0] == '\n') {
      if (!words.empty()) {
        // Create document with tokens.
        Store local(&commons);
        Document document(&local);
        for (const string &word : words) {
          document.AddToken(word);
        }

        // Parse document.
        if (!FLAGS_parser.empty()) {
          parser.Parse(&document);
        }
        document.Update();

        // Write document to output file.
        if (!FLAGS_output.empty()) {
          string fn = FLAGS_output + "/sent-" + std::to_string(sentno++);
          FileEncoder out(&local, fn);
          out.Encode(document.top());
          CHECK(out.Close());
        } else {
          std::cout << ToText(document.top()) << "\n";
        }
      }
      words.clear();
    } else {
      size_t sp = line.find(' ');
      CHECK(sp != string::npos) << line;
      line.resize(sp);
      if (line != "-DOCSTART-") words.push_back(line);
    }
  }

  return 0;
}

